import os
from flask import Flask, render_template
from tcw.config import Development, Production
from tcw.database import init_db


BASEDIR = os.path.abspath(os.path.dirname(__file__))


def create_app(name='tiny contest winners', config=None):
    """
    Create the Flask app

    args:
        - name
        - config object
    """

    if config is None:
        config = Development

    app = Flask(name, root_path=BASEDIR)
    configure_app(app, config)
    load_filters(app)
    load_handlers(app)
    load_blueprints(app)
    load_views()

    return app


def configure_app(app, config):
    """
    Configure the flask app

    args:
        - Flask app instance
        - config object
    """

    init_db(config.SQLALCHEMY_DATABASE_URI, config.DEBUG)
    app.config.from_object(config)


def load_blueprints(app):
    """
    Load flask blueprints from web apps

    args:
        - Flask app instance
    """

    from tcw.apps.contest.views import bp as contest

    app.register_blueprint(contest)


def load_views():
    """
    Load views from any of the web apps
    """

    from tcw.apps.contest import views


def load_filters(app):
    """
    Load custom filters into the apps jinja env

    args:
        - Flask app instance
    """

    from tcw.utils import md_to_html
    app.jinja_env.filters['markdown'] = md_to_html


def page_not_found(e=None):
    return render_template('404.html')


def page_expired(e=None):
    return render_template('410.html')


def load_handlers(app):
    app.register_error_handler(404, page_not_found)
    app.register_error_handler(410, page_expired)



import os

class BaseConfig:
    ## flask options ##
    PROJECT = 'tiny contest winners'
    DEBUG = False
    TESTING = False
    PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))
    SECRET_KEY = os.getenv('FLASK_SECRET', 'TerribleLifeChoices')
    WFT_SECRET_KEY = os.getenv('FLASK_SECRET', 'TerribleLifeChoices')
    WTF_CSRF_ENABLED = True
    CSRF_ENABLED = True

    ## database options ##
    SQLALCHEMY_DATABASE_URI = os.getenv('SQLALCHEMY_DATABASE_URI', 'sqlite:////tmp/test.db')
    SQLALCHEMY_ECHO = os.getenv('SQLALCHEMY_ECHO', False)
    SQLALCHEMY_TRACK_MODIFICATIONS = False

    ## recaptcha options ##
    RECAPTCHA_USE_SSL = os.getenv('RECAPTCHA_USE_SSL', True)
    RECAPTCHA_PUBLIC_KEY = os.getenv('RECAPTCHA_PUBLIC_KEY', None)
    RECAPTCHA_PRIVATE_KEY = os.getenv('RECAPTCHA_PRIVATE_KEY', None)
    RECAPTCHA_OPTIONS = os.getenv('RECAPTCHA_OPTIONS', {'theme': 'white'})


class Development(BaseConfig):
    DEBUG = True
    TESTING = True
    CSRF_ENABLED = False
    WTF_CSRF_ENABLED = False
    SQLALCHEMY_ECHO = True
    SQLALCHEMY_TRACK_MODIFICATIONS = True


class Production(BaseConfig):
    # SERVER_NAME = os.getenv('FLASK_SERVER_NAME', 'localhost')
    pass



from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker, create_session
from sqlalchemy.ext.declarative import declarative_base

# globals #
engine = None
session = scoped_session(lambda: create_session( bind=engine,
                                                    autocommit=False,
                                                    autoflush=False ))

Base = declarative_base()
Base.query = session.query_property()


def init_db(uri, echo=False):
    """
    Initialize the database.

    args:
        - db uri
        - sqlalchemy echo?
    """

    init_engine(uri, echo=echo)
    init_tables()


def init_engine(uri, *args, **kwargs):
    """
    initialize the global engine for sessions.

    args:
        - db uri
        - keyword args (see sqlalchemy create_engine() docs)
    """

    global engine
    engine = create_engine(uri, **kwargs)


def init_tables():
    """
    Initialize database tables for app models.
    """

    from tcw.apps.contest.models import Contest, Entrant
    Base.metadata.create_all(bind=engine)



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='tcw',
    version='0.0.11',
    author='J Leary',
    author_email='tinycontestwinners@gmail.com',
    description='tiny contest winners application',
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'flask',
        'flask-wtf',
        'sqlalchemy',
        'markdown',
        'gunicorn',
    ],
)



import datetime
import logging
from flask import (Blueprint, render_template, redirect, request, abort,
    url_for, Response)
from sqlalchemy.exc import IntegrityError
from tcw.database import session
from tcw.utils import (contest_by_name, fossil_by_name, random_name,
    expires_time)
from .forms import ContestForm, SignupForm
from .models import Contest, Entrant


logger = logging.getLogger(__name__)
bp = Blueprint('contest', __name__, template_folder='templates')


@bp.route('/', methods=['GET'])
def index():
    return render_template('contest/index.html')


@bp.route('/about', methods=['GET'])
def about():
    return render_template('contest/about.html')


@bp.route('/privacy', methods=['GET'])
def privacy():
    return render_template('contest/privacy.html')


@bp.route('/contest', methods=['GET', 'POST'])
def new():
    form = ContestForm()
    if form.validate_on_submit():
        unique = random_name()
        options = {
            'name': unique,
            'title': form.title.data,
            'instructions': form.instructions.data,
            'email': form.email.data,
            'expires': expires_time(float(form.hours.data)),
            'winners': int(form.winners.data),
            'max_entrants': int(form.maximum.data),
        }

        try:
            obj = Contest(**options)
            session.add(obj)
            session.commit()
            options['expires'] = options['expires'].isoformat() + "Z"
        except Exception as x:
            logger.warning(x)
            abort(404)

        return redirect(url_for('contest.success', **options))

    return render_template('contest/form.html', form=form)


@bp.route('/signup', methods=['GET', 'POST'])
def signup():
    form = SignupForm()

    try:
        name = request.args.get('name')
    except Exception as x:
        logger.warning(x)
        abort(404)

    try:
        contest = contest_by_name(name)
    except:
        try:
            fossil = fossil_by_name(name)
        except:
            abort(404)
        abort(410)

    if form.validate_on_submit():
        if len(contest.entrants) >= contest.max_entrants:
            return render_template('contest/signup.html',
                data=contest, form=form)

        options = {
            'name': form.name.data.split("|")[0],
            'contest_id': contest.id }
        try:
            obj = Entrant(**options)
            session.add(obj)
            session.commit()
            return redirect(
                url_for('contest.thanks',
                    contest=contest.title,
                    entrant=form.name.data), code=303)
        except IntegrityError:
            session.rollback()
            return render_template('contest/sorry.html',
                contest=contest, form=form)
        except Exception as x:
            logg
            abort(404)
    else:
        return render_template('contest/signup.html',
            contest=contest, form=form)


@bp.route('/success', methods=['GET'])
def success():
    return render_template('contest/success.html', data=request.args)


@bp.route('/thanks', methods=['GET'])
def thanks():
    contest = request.args.get('contest')
    entrant = request.args.get('entrant')
    return render_template('contest/thanks.html',
        contest=contest, name=entrant)
